% Elizabeth Wig 
% EE 367 final project, March 10 2022

% README: 
% Contact: ewig [at] stanford [dot] edu
% Note: you can make this public if you want, but it requires InSAR images
% processed with my group's internal InSAR processing codes, so it probably
% would not be very useful to publish on the 367 website

% This runs with only MATLAB libraries and no additional functions, but
% does require very specific InSAR output files. 
% To run this, you can open MATLAB and hit "Run"
% You need the following listed files in the same directory as this one or
% else in a subfolder with the listed number of looks from InSAR processing
% (e.g. the folder is called "10looks")

% Needs files: 
% parameters (lists number of points in range and azimuth of acquisition, 
  % plus number of interferograms)
% jdlist (list of Julian dates that SAR images were acquired)
% tripleunwlist (list of unwrapped phase closure triplet files)
    % These are the actual images that get filtered! They are read in as
    % the variable "phase" and image processed etc. 
% elevation.dem.rsc (file writing the spatial coordinates in
  % latitude/longitude and spacing of the output files)
% And a time series of .unw (unwrapped triplet) files whose file names are 
  % listed in tripleunwlist
clear; 

% for 10 looks use minimum nav_area=4 or with 30 looks nav_area 3
nlooks = 10; % Number of looks the unwrapped images you're loading have
nav_area = 5; % Area to average over
restoredefaultpath; % get default MATLAB path to load so we can find files
% close all;

% Loads the appropriate file dimensions based on number of looks nlooks
% Also gets the correct latitude/longitude coordinates
% Here nlooks = 10, for all processing purposes.
if nlooks == 5
    % The information here on latitude/longitude is copied over from the
    % elevation.dem.rsc file
    WIDTH        = 2819;%  720;   
    FILE_LENGTH  = 2056;% 720;
    X_FIRST      =  -99.23778;
    Y_FIRST      =  36.94111;
    X_STEP       =  0.0027777778/2;
    Y_STEP       =  -0.0027777778/2;
    addpath 5looks; 
elseif nlooks == 10
    WIDTH        = 1409;%  720;
    FILE_LENGTH  = 1028;% 720;
    X_FIRST      =  -99.23778;
    Y_FIRST      =  36.94111;
    X_STEP       =  0.0027777778;
    Y_STEP       =  -0.0027777778;
    addpath 10looks;
elseif nlooks == 15
    WIDTH        = 939;%  720;
    FILE_LENGTH  = 685;% 720;
    X_FIRST      =  -99.23778;
    Y_FIRST      =  36.94111;
    X_STEP       =  0.0027777778*3/2;
    Y_STEP       =  -0.0027777778*3/2;
    addpath 15looks;
elseif nlooks == 20
    WIDTH        = 704;%  720;
    FILE_LENGTH  = 514;% 720;
    X_FIRST      =  -99.23778;
    Y_FIRST      =  36.94111;
    X_STEP       =  0.0027777778*2;
    Y_STEP       =  -0.0027777778*2;
    addpath 20looks;
elseif nlooks == 30
    WIDTH        = 469;%  720;
    FILE_LENGTH  = 343;% 720;
    X_FIRST      =  -99.23778;
    Y_FIRST      =  36.94111;
    X_STEP       =  0.0027777778*3;
    Y_STEP       =  -0.0027777778*3;
    addpath 30looks; 
elseif nlooks == 50
    WIDTH        = 281;%  720;
    FILE_LENGTH  = 205;% 720;
    X_FIRST      =  -99.23778;
    Y_FIRST      =  36.94111;
    X_STEP       =  0.0027777778*5;
    Y_STEP       =  -0.0027777778*5;
    addpath 50looks; 
end
    x_steps = X_FIRST:X_STEP:X_FIRST+X_STEP*WIDTH;
    y_steps = Y_FIRST:Y_STEP:Y_FIRST+Y_STEP*FILE_LENGTH;

% Load some parameters from the file: the number of points in range and
% azimuth, plus the number of single-look complex files and the number of
% interferograms to load
load 'parameters'
nr=parameters(1);
naz=parameters(2);
nslcs=parameters(3);
nigrams=parameters(4);

% Loads Julian Dates (JDs) of InSAR acquisitions
jd=load('jdlist');
t_closure=datetime(jd,'convertfrom','juliandate','Format','ddMMMyy');

% more params
lambda=0.0566; % Wavelength

% I used to do some filtering here, but that's obsolete. I set this to 1
% for no filtering but kept the code here in case I want to use that option
% again.
spatialfilter=1; % 5 % Set to 1 for no filtering. Previously set to 5
temporalfilter=1; % 5 % set to 1 for no filtering. previously set to 5

% read in the unwrapped triplet files
cells=importdata('tripleunwlist'); % List of unwrapped triplets
% ints = importdata('intlist'); % if we wanted wrapped files

% Initialize variables to be loaded
N=length(cells);
phase=zeros(nr,naz,N);
amp=zeros(nr,naz);
dat=zeros(2*nr,naz);

% "IF" statement about whether we've run the bilateral, nlm, DnCNN filters
% I put this "if" in so we can load the pre-filtered version of the phase matrix
% since it made my computer sound like it was launching into outer space
% If it's already run and you saved it as a .mat file, we load the file.
% Otherwise, we'll run the filters again as part of the data loading loop.
weAlreadyRanFilters =  1; 

if weAlreadyRanFilters == 1
    % Load data we already have (I saved the optimal choice after running
    % filters)
    load('phase_bilat.mat'); 
    load('phase_nlm.mat');
    load('phase_dncnn.mat');
else 
    % Establish degree of smoothing for bilateral, nonlocal means
    bilat_filt_param = 100000; nlm_filt_param = 3;
    % Initialize empty arrays 
    phase_bilat = zeros(nr,naz,N); phase_nlm = zeros(nr,naz,N);
    phase_dncnn = zeros(nr,naz,N);  
    % Tell the code to load the DnCNN denoising network 
    dncnnnet = denoisingNetwork('dncnn');
end

% For loop to load each file of phase triplets in time series. 
% Each "center date" has its own file. There are 83, forms an MxNx83 array
for i=1:N 
    strunw=cells{i};
    filename_a=sprintf('%s',strunw);

    fid=fopen(filename_a);
    dat=fread(fid,[2*nr,inf],'float','ieee-le');
    amp=amp+dat(1:nr,:).^0.05;

    % currently doing no filtering here with medfilt2
    % - spatialfitler=1 and the median filter doesn't do anything
    % the medfilt2 is left for posterity/in case I want to resurrect it
    phase(:,:,i)=medfilt2(mod(dat(nr+1:nr*2,:)+pi,2*pi)-pi,[spatialfilter spatialfilter]); 

    % If we didn't already run filters, filter each InSAR phase closure
    % image with the biteral, NLM, and DnCNN filters.
    if weAlreadyRanFilters == 0
        phase_bilat(:,:,i) = imbilatfilt(phase(:,:,i),bilat_filt_param); % Default degree of smoothing was 0.1. We wanted a little less.
        phase_nlm(:,:,i) = imnlmfilt(phase(:,:,i), 'DegreeOfSmoothing',nlm_filt_param); % original estDoS was 0.2765. We wanted a little less. 
        phase_dncnn(:,:,i) = denoiseImage(phase(:,:,i),dncnnnet);
    end
    fclose(fid);

    % Optionally loading InSAR correlation files - commented out here since
    % it's not used in this code. 
%     filename_cc_tr = strcat(strunw(1:24), '.cc');
%     fid = fopen(filename_cc_tr);
%     ccdat=fread(fid,[2*nr,inf],'float','ieee-le');
%     cc_vals(:,:,i) = ccdat(nr+1:nr*2,:);
%     fclose(fid);

end
disp('Phases and amplitudes read in.');

%% Put in locations of 11 ground truth soil moisture measurement sites
locstr = ['acme';'chan';'fair';'skia';'hold';'minc';'king';'redr';'wash';'hint';'okmu']; 
location_str = ["Acme","Chandler","Fairview","Skiatook","Holdenville","Minco","Kingfisher","Red Rock","Washington","Hinton","Okmulgee"];
xlocs = [-98.023250,-96.804070, -98.497660, -96.037060, -96.355950,-97.955530,-97.954420,-97.153060,-97.521090,-98.481510, -95.914730];
ylocs = [ 34.808330, 35.652820,  36.263530,  36.415300,  35.070730, 35.272250, 35.854310, 36.355900, 34.982240, 35.484390,  35.582110];

%% "For" loop which does a few things:
% (a) read in soil moisture data
% (b) spatially average (or use denoised version) of InSAR data
% (c) Get the predicted soil moisture parameter from InSAR data and 
% (d) Determine how well the soil moisture and InSAR data are correlated

M = 20; % temporal averaging over 2M+1 days. 
K = length(locstr); % Number of sites (11)

% Initialize arrays that will update in loop
mat1 = zeros(1,11); mat2 = zeros(4,11);
pmat1 = mat1; pmat2 = mat2;
x = zeros(size(xlocs)); y = zeros(size(ylocs));
vr_av = zeros(N,K); v_av = zeros(N,K); v_med = zeros(N,K);
total_soil_av = zeros(N,K); 

for k = 1:K % across all K (11) locations
    % First, read in soil moisture data downloaded from Oklahoma Mesonet
    T2 = readtable(strcat(locstr(k,:),'-moisture-2018-2021.csv'),'NumHeaderLines',1);
    soil_days = [strcat(T2.Var1,'-2018'); strcat(T2.Var1,'-2019'); strcat(T2.Var1(1:59),'-2020'); {'Feb 29-2020'}; strcat(T2.Var1(60:end),'-2020'); strcat(T2.Var1,'-2021')];
    total_soil = [T2.Var2; T2.Var3; T2.Var4(1:59); NaN; T2.Var4(60:end); T2.Var5];
    t_meso = datetime(soil_days,'InputFormat','MMM dd-yyyy');
    
   % Find the location in the InSAR image that the soil moisture site is at
    x(k) = find(abs(x_steps-xlocs(k))<0.0027777778*nlooks/20);
    y(k) = find(abs(y_steps-ylocs(k))<0.0027777778*nlooks/20);

    % Find the phase closure values at that site
    v=squeeze(squeeze(phase(x(k),y(k),:))); % unfiltered (use for median/mean)
    v_bilat(:,k) = squeeze(squeeze(phase_bilat(x(k),y(k),:))); % Bilateral-filtered phase closure at site
    v_nlm(:,k) = squeeze(squeeze(phase_nlm(x(k),y(k),:))); % NLM-filtered phase closure at site
    v_dncnn(:,k) = squeeze(squeeze(phase_dncnn(x(k),y(k),:))); % DnCNN-filtered phase closure at site
    
    %% Closure integration averaging over area
    n = nav_area; % 2*n+1 is the area we average/median over
    if n > 0
        % Here we compute the mean and median locally just at the site
        % (rather than across the whole image) just because it's faster
        v_av(:,k) = squeeze(squeeze(mean(mean(phase(x(k)-n:x(k)+n,y(k)-n:y(k)+n,:)))));
        v_med(:,k) =squeeze(squeeze(median(median(phase(x(k)-n:x(k)+n,y(k)-n:y(k)+n,:)))));
    else
        v_av(:,k) = v;
        v_med(:,k) = v;
    end

    % Take a time integral of the phase at each point across the time
    % series of 83 different images acquired on different days
    % Then use polyfit to make a fit line to the integrated phase closure
    % and subtract the bias
    % The bias is called "rw" here. This part of the processing is what I
    % presented at a conference in December - the bias removal. Not super
    % relevant to this project, just a processing step we do to all images.
    vi_av = cumsum(v_av(:,k));
    %vi_med = cumsum(v_med(:,k));
    xp = linspace(0,1,length(t_closure)-2)';
    yp = polyfit(sqrt(xp), cumsum(v_av(:,k)), 1);

    yp_med = polyfit(sqrt(xp),cumsum(v_med(:,k)),1);
    yp_bilat = polyfit(sqrt(xp),cumsum(v_bilat(:,k)),1);
    yp_nlm = polyfit(sqrt(xp),cumsum(v_nlm(:,k)),1);
    yp_dncnn = polyfit(sqrt(xp), cumsum(v_dncnn(:,k)),1);
    
    rw = sqrt(xp)*yp(1)+yp(2);
    rw_med = sqrt(xp)*yp_med(1) + yp_med(2);
    rw_bilat = sqrt(xp)*yp_bilat(1)+yp_bilat(2);
    rw_nlm = sqrt(xp)*yp_nlm(1)+yp_nlm(2);
    rw_dncnn = sqrt(xp)*yp_dncnn(1)+yp_dncnn(2);

    % figure; hold on; plot(t_closure(2:end-1), cumsum(v_av)-rw,'LineWidth',2); ylabel('Integrated Phase Closure')
    % yyaxis right; plot(t_soil,total_soil); ylabel('Soil Moisture Saturation (%)');
    % title('Integrated Closure vs Soil Saturation','FontSize',14)

    % Take average of soil moisture over time in days surrounding the InSAR
    % acquisitions. For example, if an InSAR triplet was acquired on May 1,
    % 13, and 25, the soil moisture data would be averaged over something
    % on the order of the whole month of May, depending on the value of M. 
    m = M;
    % m = 12; % m = 5 % number of days to average over +/-
    t_meso_ind = zeros(2*m+1,length(t_closure)-2);
    for p = 2:length(t_closure)-1 % 2:length(t_closure)-1
        %     t_meso_ind(:,n-1) = find(abs(t_meso - t_closure(n)) <= m);
        t_meso_ind(:,p-1) = find(abs(t_meso - t_closure(p)) <= m)';
    end

    total_soil_av(:,k) = mean(total_soil(t_meso_ind),'omitnan');
    vr_av(:,k) = cumsum(v_av(:,k))-rw;
    vr_med(:,k) = cumsum(v_med(:,k))-rw_med;
    vr_bilat(:,k) = cumsum(v_bilat(:,k))-rw_bilat;
    vr_nlm(:,k) = cumsum(v_nlm(:,k))-rw_nlm;
    vr_dncnn(:,k) = cumsum(v_dncnn(:,k))-rw_dncnn;

    % Compute the correlation coefficient between the InSAR-derived
    % parameter and the soil moisture parameter. 
    % Correlation coefficient is used to quantitatively measure how well
    % the fit is for the different methods. 
    [Rs_int,Ps_int] = corrcoef(total_soil_av(4:end-1,k), vr_av(4:end-1,k),'Rows','complete');
    [Rs_int_med,Ps_int_med] = corrcoef(total_soil_av(4:end-1,k), vr_med(4:end-1,k),'Rows','complete');
    [Rs_int_bilat,Ps_int_bilat] = corrcoef(total_soil_av(4:end-1,k),vr_bilat(4:end-1,k),'Rows','complete');
    [Rs_int_nlm,Ps_int_nlm] = corrcoef(total_soil_av(4:end-1,k),vr_nlm(4:end-1,k),'Rows','complete');
    [Rs_int_dncnn,Ps_int_dncnn] = corrcoef(total_soil_av(4:end-1,k),vr_dncnn(4:end-1,k),'Rows','complete');
    [Rs_raw,Ps_raw] = corrcoef(total_soil_av(:,k),v_av(:,k),'Rows','complete');

    % Create a matrix of correlation coefficients for each method. The
    % matrix contains the correlation coefficients at each site. 
    mat1(:,k) =Rs_int(1,2);
    pmat1(:,k)=Ps_int(1,2);
    mat1med(:,k) = Rs_int_med(1,2);
    mat1bilat(:,k) = Rs_int_bilat(1,2);
    mat1nlm(:,k) = Rs_int_nlm(1,2);
    mat1dncnn(:,k) = Rs_int_dncnn(1,2);
        %end
end
% Show the average correlation coefficient for all methods (mean, median,
% Bilateral, NLM, DnCNN)
disp(mean(mat1))
disp(mean(mat1med))
disp(mean(mat1bilat))
disp(mean(mat1nlm))
disp(mean(mat1dncnn))

%% Make plots of the time series! 
plotClosures = 1;
if plotClosures == 1

% Pick some nice RGB colors to use
darkGray = [77 79 83]/255; 
darkPurple = [83 40 79]*1.2/255; lightPurple = [179 98 172]/255; 
darkTeal = [0 80 92]/255; mediumTeal = [0 171 196]/255;
meddarkTeal = [0 116 134]/255;
cardinalRed = [140 21 21]/225; lightRed = [140 51 51]/140; 
blue = [0 50 195]/255; 

% Not relevant to EE367 project - plot original phase closure without
% integrating or removing bias
fig=figure; set(fig,'defaultAxesColorOrder',[cardinalRed; blue]);
yyaxis left; plot(t_closure(5:end-1), v_av(4:end,4),'LineWidth',2,'Color',cardinalRed); 
ylabel('Phase Closure','FontSize',20,'FontName','Segoe UI'); ylim([-2 2]);
hold on; yyaxis right; plot(t_closure(5:end-1),100*total_soil_av(4:end,4),'LineWidth',2,'Color',blue); ylabel('Soil Moisture %','FontSize',16,'FontName','Segoe UI'); ylim([0 100]); %'-s','MarkerSize',4,
legend('Phase Closure','Soil Moisture %','FontSize',14,'Location','SouthEast','FontName','Segoe UI');
title('Skiatook, OK: Closure & Soil Moisture over Time','FontSize',24,'FontName','Segoe UI'); 
ax = gca; ax.FontSize=16; ax.FontName = 'Segoe UI';

% Relevant to EE367 project - plot InSAR-derived parameter from all 5
% methods along with ground truth, and compare. This parameter should match
% soil moisture but increase when it decreases. It's used to generate the
% estimate.
fig2=figure; set(fig2,'defaultAxesColorOrder',[cardinalRed; blue]);
yyaxis left;
hold on; plot(t_closure(5:end-1), vr_av(4:end,4),'LineWidth',2,'Color',cardinalRed); ylim([-4 4]); %'-d','MarkerSize',4, % ylim([-2.5 2.5])
plot(t_closure(5:end-1),vr_med(4:end,4),'LineWidth',2,'Color',meddarkTeal); 
plot(t_closure(5:end-1),vr_bilat(4:end,4),'LineWidth',2,'Color',darkPurple); 
plot(t_closure(5:end-1),vr_nlm(4:end,4),'LineWidth',2,'Color',darkPurple); 
plot(t_closure(5:end-1),vr_dncnn(4:end,4),'LineWidth',2,'Color',darkPurple);
ylabel('Phase Closure')
hold on; yyaxis right; plot(t_closure(5:end-1),100*total_soil_av(4:end,4),'LineWidth',2,'Color',blue); ylabel('Soil Moisture %','FontSize',16,'FontName','Segoe UI'); ylim([0 100]); %'-s','MarkerSize',4,
legend('Closure Mean','Closure Median','Closure Bilateral','Closure Nonlocal','Closure DnCNN','Soil Moisture %','FontSize',14,'Location','SouthEast','FontName','Segoe UI');
title('Skiatook, OK: Integrated Closure & Soil Moisture over Time','FontSize',24,'FontName','Segoe UI'); 
ax = gca; ax.FontSize=16; ax.FontName = 'Segoe UI';

end
%%
% Relevant to EE367 project - plot estimated soil moisture from all 5
% methods along with ground truth, and compare. 
% Also makes the scatter plot shown in the final report
plotScatter = 1;
if plotScatter == 1
    total_soil_av(total_soil_av >= 1) = 0.9999;
    for k = 4%1:11 % Can pick sites out of 11
        figure; semilogy(vr_av(4:end,k),1-total_soil_av(4:end,k),'o','LineWidth',2,'Color',cardinalRed);
        yticks([0 .01 .1 .2 .4 .7 1]); yticklabels({'100%','99%','90%','80%','60%','30%','0%'});
        [logfit, S] = polyfit(vr_av(4:end,k), log(1-total_soil_av(4:end,k)),1);
        [logfit_med,S_med] = polyfit(vr_med(4:end,k),log(1-total_soil_av(4:end,k)),1);
        [logfit_bi,S_bi] = polyfit(vr_bilat(4:end,k),log(1-total_soil_av(4:end,k)),1);
        [logfit_nlm,S_nlm] = polyfit(vr_nlm(4:end,k),log(1-total_soil_av(4:end,k)),1);
        [logfit_dncnn,S_dncnn] = polyfit(vr_dncnn(4:end,k),log(1-total_soil_av(4:end,k)),1);

        soil_pred = logfit(1).*vr_av(4:end,k) + logfit(2);
        soil_pred_med = logfit_med(1)*vr_med(4:end,k)+logfit_med(2);
        soil_pred_bi = logfit_bi(1)*vr_bilat(4:end,k)+logfit_bi(2);
        soil_pred_nlm = logfit_nlm(1)*vr_nlm(4:end,k)+logfit_nlm(2);
        soil_pred_dncnn = logfit_dncnn(1)*vr_dncnn(4:end,k)+logfit_dncnn(2);

        [ylog, delta] = polyval(logfit,vr_av(4:end-4),S);
        %     [soil_pred,delta]  = polyval(logfit,vr_av(4:end,4),S);
        hold on; semilogy(vr_av(4:end,k), exp(soil_pred),'LineWidth',2,'Color','k');
        
        xlabel('Integrated Phase Closure Minus Random Walk','FontSize',16,'FontName','Segoe UI');
        ylabel('Soil Moisture','FontSize',16,'FontName','Segoe UI');
        title(strcat(location_str(k),': Soil Moisture vs. Phase Closure'),'FontSize',16,'FontName','Segoe UI')
        ax = gca; ax.FontSize=16; ax.FontName = 'Segoe UI';
        
        figure; plot(t_closure(5:end-1), 100*total_soil_av(4:end,k),'LineWidth',2,'Color',blue);
        hold on; plot(t_closure(5:end-1), 100-100*exp(soil_pred),'LineWidth',2,'Color',cardinalRed);
        hold on; plot(t_closure(5:end-1), 100-100*exp(soil_pred_med),'LineWidth',2,'Color',meddarkTeal);
        hold on; plot(t_closure(5:end-1), 100-100*exp(soil_pred_bi),'LineWidth',2,'Color',darkPurple);
        hold on; plot(t_closure(5:end-1), 100-100*exp(soil_pred_nlm),'LineWidth',2,'Color',lightPurple);
        hold on; plot(t_closure(5:end-1), 100-100*exp(soil_pred_dncnn),'LineWidth',2,'Color',darkGray);

        ylabel('Soil Moisture %'); 
        title(strcat(location_str(k),': Measured vs. Predicted Soil Moisture'));
        legend('Measured Soil Moisture','Predicted: Mean','Predicted: Median','Predicted: Bilateral','Predicted: NLM','Predicted: DnCNN','Location','SouthEast');
        ax = gca; ax.FontSize=16; ax.FontName = 'Segoe UI';
    end
end

mat1means = [mean(mat1),mean(mat1med),mean(mat1bilat),mean(mat1nlm),mean(mat1dncnn)];
mat1stds = [std(mat1),std(mat1med),std(mat1bilat),std(mat1nlm),std(mat1dncnn)];
figure; errorbar(categorical({'1. Mean','2. Median','3. Bilateral','4. NLM','5. DnCNN'}),abs(mat1means),mat1stds,'o','LineWidth',2);
ax = gca; ax.FontSize = 16; ax.FontName= 'Segoe UI'; 
title('Average Correlation between Soil Moisture and InSAR')
ylabel('Avg. Correlation, Std. Dev. Error Bars');
xlabel('Denoising Type')
%% Show station locations & correlations superimposed on various filtered images
showStaLocs = 1;
if showStaLocs == 1
phasetypes(:,:,1) = filter2(ones(2*nav_area+1)/(2*nav_area+1)^2,phase(:,:,1));
phasetypes(:,:,2) = medfilt2(phase(:,:,1),[2*nav_area+1,2*nav_area+1]);
phasetypes(:,:,3) = phase_bilat(:,:,1);
phasetypes(:,:,4) = phase_nlm(:,:,1);
phasetypes(:,:,5) = phase_dncnn(:,:,1);
corrmat = [mat1; mat1med; mat1bilat; mat1nlm; mat1dncnn];
typenames = ["Linear Mean","Median","Bilateral","NLM","DnCNN"];
    for k = 1:5
        img1 = phasetypes(:,:,k)'; img1(amp'<50) = -pi; % Filter out the edges
        figure; imagesc(x_steps,y_steps,img1); colormap gray; set(gca,'YDir','normal');
        fillGradient = abs(corrmat(k,:));
        fillColors = hsv2rgb([ones(1,11); fillGradient;ones(1,11)]');
        hold on; scatter(xlocs,ylocs,200,fillColors,'filled');
        text(xlocs+.05,ylocs, ["Acme","Chandler","Fairview","Skiatook","Holdenville","Minco","Kingfisher","Red Rock","Washington","Hinton","Okmulgee"], 'HorizontalAlignment','left','FontName','Segoe UI')
        clrmap =  hsv2rgb([ones(1,21); 0:0.05:1;ones(1,21)]');
        title(strcat(typenames(k),': Filtered Image with R (correlation) between InSAR and Soil Moisture'));
        xlabel('Longitude (deg)'); ylabel('Latitude (deg)')
        ax = gca; ax.FontSize=14; ax.FontName = 'Segoe UI';
        % colormap(clrmap); colorbar;
        caxis([-pi pi])
    end
end

%% Show parameter sweep of NLM and Bilateral filters
% I just did this manually and collected the values in a table
% Could have run more systemically I suppose, but wanted to have OK
% processing time so I selected parameters each time to get a good sweep
nlmparams = [0.01,  0.1,   0.5,   1,      3,     5,      7.5,    10,    15,     20,   50];
nlmvalues = [0.0505,0.0505, 0.0439,0.0439,0.3203,0.3202, 0.3194,0.3190,0.3187,0.3186,0.3185];
bilparams = [0.001, 0.01,  0.1,    1,    10,    20,    50,    100,   1000, 10000,100000];
bilvalues = [0.0507,0.0491,0.0520,0.0534,0.1272,0.1415,0.1497,0.1522,0.1544,0.1546,0.1546];

figure;  semilogx(bilparams,bilvalues); hold on; 
loglog(nlmparams,nlmvalues);
title('Parameter Sweep');
legend('Bilateral','Non-Local Means')
